/*
 * dlgapenf.c --- append font dialog
 * 
 * Copyright (c) 2019 Mariano Alvarez Fernandez (malfer@telefonica.net)
 *
 * This file is part of FEdit, a visual font editor for the MGRX font format
 *
 * FEdit is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FEdit is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FEdit. If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <grgui.h>
#include <mgrxcolr.h>
#include "fedit.h"

#define COMMAND_OK           1
#define COMMAND_CANCEL       2
#define COMMAND_CF           3

static GUIDialog *fapendlg = NULL;
static GUIGroup *fapengo = NULL;

static int process_fapendlg_event(void *udata, GrEvent *ev)
{
    char *s;

    GUIDialog *d = (GUIDialog *)udata;
    GUIGroup *go = (GUIGroup *)(d->exdata);

    if (ev->type == GREV_COMMAND) {
        switch (ev->p1) {
            case COMMAND_OK :
                d->result = 1;
               return -1;
            case COMMAND_CANCEL :
                d->result = -1;
                return -1;
            case COMMAND_CF :
                FileChooseDlgCreate("Choose Font File to Append From", 480, 320);
                s = GUIGroupGetText(go, 1, GrGetChrtypeForUserEncoding());
                FileChooseDlgSetData(s);
                free(s);
                if (FileChooseDlgRun() == 1) {
                    s = malloc(MAXLEN_FNAME+1);
                    FileChooseDlgGetData(s);
                    GUIGroupSetText(go, 1, s);
                    GUIGroupRePaintObject(go, 1);
                    free(s);
                }
                FileChooseDlgDestroy();
                return 1;
        }
    }

    return GUIGroupProcessEvent(go, ev);
}

int FApenDlgCreate(void)
{
    if (fapendlg) return 1; // dialog already created

    fapengo = GUIGroupCreate(15, 0, 0);
    if (fapengo == NULL) return 0;
    GUIObjectSetLabel(&(fapengo->o[0]), 0,   0,   0,  80, 30, GrNOCOLOR, WEBC_WHITE, "Font name");
    GUIObjectSetEntry(&(fapengo->o[1]), 1,  80,   0, 300, 30, WEBC_WHITE, WEBC_BLACK, MAXLEN_FNAME, "");
    GUIObjectSetButton(&(fapengo->o[2]), 2,380,   0,  20, 30, WEBC_WHITE, WEBC_BLACK, ".", COMMAND_CF, 0, 0);
    GUIObjectSetLabel(&(fapengo->o[3]), 3,   0,  40, 160, 30, GrNOCOLOR, WEBC_WHITE, "First source glyph");
    GUIObjectSetEntry(&(fapengo->o[4]), 4, 160,  40, 240, 30, WEBC_WHITE, WEBC_BLACK, 8, "");
    GUIObjectSetLabel(&(fapengo->o[5]), 5,   0,  80, 160, 30, GrNOCOLOR, WEBC_WHITE, "Last source glyph");
    GUIObjectSetEntry(&(fapengo->o[6]), 6, 160,  80, 240, 30, WEBC_WHITE, WEBC_BLACK, 8, "");
    GUIObjectSetLabel(&(fapengo->o[7]), 7,   0, 120, 160, 30, GrNOCOLOR, WEBC_WHITE, "First dest. glyph");
    GUIObjectSetEntry(&(fapengo->o[8]), 8, 160, 120, 240, 30, WEBC_WHITE, WEBC_BLACK, 8, "");
    GUIObjectSetLabel(&(fapengo->o[9]), 9,   0, 160, 160, 30, GrNOCOLOR, WEBC_WHITE, "X origin");
    GUIObjectSetEntry(&(fapengo->o[10]), 10, 160, 160, 240, 30, WEBC_WHITE, WEBC_BLACK, 8, "");
    GUIObjectSetLabel(&(fapengo->o[11]), 11,   0, 200, 160, 30, GrNOCOLOR, WEBC_WHITE, "Y origin");
    GUIObjectSetEntry(&(fapengo->o[12]), 12, 160, 200, 240, 30, WEBC_WHITE, WEBC_BLACK, 8, "");
    GUIObjectSetButton(&(fapengo->o[13]), 13,  50, 250, 140, 40, WEBC_GREEN, WEBC_WHITE, "Ok", COMMAND_OK, 0, 0);
    GUIObjectSetButton(&(fapengo->o[14]), 14, 210, 250, 140, 40, WEBC_GREEN, WEBC_WHITE, "Cancel", COMMAND_CANCEL, 0, 0);
    GUIGroupSetSelected(fapengo, 1, 0);

    fapendlg = GUIGroupDialogCreate("Append glyphs from Font",
                                    fapengo, process_fapendlg_event);
    if (fapendlg == NULL) {
        GUIGroupDestroy(fapengo);
        return 0;
        fapengo = NULL;
    }

    return 1;
}

void FApenDlgDestroy(void)
{
    if (fapendlg) {
        GUIDialogDestroy(fapendlg);
        fapendlg = NULL;
    }
    if (fapengo) {
        GUIGroupDestroy(fapengo);
        fapengo = NULL;
    }
}

void FApenDlgSetData(char *output, int fsglyph, int lsglyph,
                     int fdglyph, int xpivot, int ypivot)
{
    char s[121];
    
    if (!fapendlg) return;

    GUIGroupSetText(fapengo, 1, output);
    sprintf(s, "%d", fsglyph);
    GUIGroupSetText(fapengo, 4, s);
    sprintf(s, "%d", lsglyph);
    GUIGroupSetText(fapengo, 6, s);
    sprintf(s, "%d", fdglyph);
    GUIGroupSetText(fapengo, 8, s);
    sprintf(s, "%d", xpivot);
    GUIGroupSetText(fapengo, 10, s);
    sprintf(s, "%d", ypivot);
    GUIGroupSetText(fapengo, 12, s);
}

void FApenDlgGetData(char *output, int *fsglyph, int *lsglyph,
                     int *fdglyph, int *xpivot, int *ypivot)
{
    char *s;

    if (!fapendlg) return;

    s = GUIGroupGetText(fapengo, 1, GrGetChrtypeForUserEncoding());
    strcpy(output, s);
    free(s);
    s = GUIGroupGetText(fapengo, 4, GrGetChrtypeForUserEncoding());
    *fsglyph = atoi(s);
    free(s);
    s = GUIGroupGetText(fapengo, 6, GrGetChrtypeForUserEncoding());
    *lsglyph = atoi(s);
    free(s);
    s = GUIGroupGetText(fapengo, 8, GrGetChrtypeForUserEncoding());
    *fdglyph = atoi(s);
    free(s);
    s = GUIGroupGetText(fapengo, 10, GrGetChrtypeForUserEncoding());
    *xpivot = atoi(s);
    free(s);
    s = GUIGroupGetText(fapengo, 12, GrGetChrtypeForUserEncoding());
    *ypivot = atoi(s);
    free(s);
}

int FApenDlgRun(void)
{
    if (!fapendlg) return -1; 

    fapendlg->result = -1;
    GUIDialogRun(fapendlg);
    return fapendlg->result;
}

