/*
 * dlgepro.c --- font properties dialog
 * 
 * Copyright (c) 2019 Mariano Alvarez Fernandez (malfer@telefonica.net)
 *
 * This file is part of FEdit, a visual font editor for the MGRX font format
 *
 * FEdit is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FEdit is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FEdit. If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <grgui.h>
#include <mgrxcolr.h>
#include "fedit.h"

#define COMMAND_OK           1
#define COMMAND_CANCEL       2

static GUIDialog *fpropdlg = NULL;
static GUIGroup *fpropgo = NULL;

static int process_fpropdlg_event(void *udata, GrEvent *ev)
{
    GUIDialog *d = (GUIDialog *)udata;
    GUIGroup *go = (GUIGroup *)(d->exdata);

    if (ev->type == GREV_COMMAND) {
        switch (ev->p1) {
            case COMMAND_OK :
                d->result = 1;
               return -1;
            case COMMAND_CANCEL :
                d->result = -1;
                return -1;
        }
    }

    return GUIGroupProcessEvent(go, ev);
}

int FPropDlgCreate(void)
{
    if (fpropdlg) return 1; // dialog already created

    fpropgo = GUIGroupCreate(16, 0, 0);
    if (fpropgo == NULL) return 0;
    GUIObjectSetLabel(&(fpropgo->o[0]), 0,   0,   0, 160, 30, GrNOCOLOR, WEBC_WHITE, "Font name");
    GUIObjectSetEntry(&(fpropgo->o[1]), 1, 160,   0, 160, 30, WEBC_WHITE, WEBC_BLACK, 16, "");
    GUIObjectSetLabel(&(fpropgo->o[2]), 2,   0,  40, 160, 30, GrNOCOLOR, WEBC_WHITE, "Font family");
    GUIObjectSetEntry(&(fpropgo->o[3]), 3, 160,  40, 160, 30, WEBC_WHITE, WEBC_BLACK, 16, "");
    GUIObjectSetLabel(&(fpropgo->o[4]), 4,   0,  80, 160, 30, GrNOCOLOR, WEBC_WHITE, "Min glyph");
    GUIObjectSetEntry(&(fpropgo->o[5]), 5, 160,  80, 160, 30, WEBC_WHITE, WEBC_BLACK, 8, "");
    GUIObjectSetLabel(&(fpropgo->o[6]), 6,   0, 120, 160, 30, GrNOCOLOR, WEBC_WHITE, "Max glyph");
    GUIObjectSetEntry(&(fpropgo->o[7]), 7, 160, 120, 160, 30, WEBC_WHITE, WEBC_BLACK, 8, "");
    GUIObjectSetLabel(&(fpropgo->o[8]), 8,   0, 160, 160, 30, GrNOCOLOR, WEBC_WHITE, "Baseline position");
    GUIObjectSetEntry(&(fpropgo->o[9]), 9, 160, 160, 160, 30, WEBC_WHITE, WEBC_BLACK, 8, "");
    GUIObjectSetLabel(&(fpropgo->o[10]), 10,   0, 200, 160, 30, GrNOCOLOR, WEBC_WHITE, "Underline height");
    GUIObjectSetEntry(&(fpropgo->o[11]), 11, 160, 200, 160, 30, WEBC_WHITE, WEBC_BLACK, 8, "");
    GUIObjectSetLabel(&(fpropgo->o[12]), 12,   0, 240, 160, 30, GrNOCOLOR, WEBC_WHITE, "Font encoding");
    GUIObjectSetEntry(&(fpropgo->o[13]), 13, 160, 240, 160, 30, WEBC_WHITE, WEBC_BLACK, 8, "");
    GUIObjectSetButton(&(fpropgo->o[14]), 14,  10, 290, 140, 40, WEBC_GREEN, WEBC_WHITE, "Ok", COMMAND_OK, 0, 0);
    GUIObjectSetButton(&(fpropgo->o[15]), 15, 170, 290, 140, 40, WEBC_GREEN, WEBC_WHITE, "Cancel", COMMAND_CANCEL, 0, 0);
    GUIGroupSetSelected(fpropgo, 1, 0);

    fpropdlg = GUIGroupDialogCreate("Edit Font properties",
                                    fpropgo, process_fpropdlg_event);
    if (fpropdlg == NULL) {
        GUIGroupDestroy(fpropgo);
        fpropgo = NULL;
        return 0;
    }

    return 1;
}

void FPropDlgDestroy(void)
{
    if (fpropdlg) {
        GUIDialogDestroy(fpropdlg);
        fpropdlg = NULL;
    }
    if (fpropgo) {
        GUIGroupDestroy(fpropgo);
        fpropgo = NULL;
    }
}

void FPropDlgSetData(FontWork *fwk)
{
    char s[121];
    
    if (!fpropdlg) return;

    GUIGroupSetText(fpropgo, 1, fwk->h.name);
    GUIGroupSetText(fpropgo, 3, fwk->h.family);
    sprintf(s, "%d", fwk->h.minchar);
    GUIGroupSetText(fpropgo, 5, s);
    sprintf(s, "%d", fwk->h.numchars + fwk->h.minchar - 1);
    GUIGroupSetText(fpropgo, 7, s);
    sprintf(s, "%d", fwk->h.baseline);
    GUIGroupSetText(fpropgo, 9, s);
    sprintf(s, "%d", fwk->h.ulheight);
    GUIGroupSetText(fpropgo, 11, s);
    sprintf(s, "%d", fwk->h.encoding);
    GUIGroupSetText(fpropgo, 13, s);
}

void FPropDlgGetData(FontWork *fwk)
{
    char *s;
    int baseline;
    int fglyph, lglyph;

    if (!fpropdlg) return;

    s = GUIGroupGetText(fpropgo, 1, GrGetChrtypeForUserEncoding());
    if (fwk->h.name) free(fwk->h.name);
    fwk->h.name = s;
    s = GUIGroupGetText(fpropgo, 3, GrGetChrtypeForUserEncoding());
    if (fwk->h.family) free(fwk->h.family);
    fwk->h.family = s;
    s = GUIGroupGetText(fpropgo, 5, GrGetChrtypeForUserEncoding());
    fglyph = atoi(s);
    s = GUIGroupGetText(fpropgo, 7, GrGetChrtypeForUserEncoding());
    lglyph = atoi(s);
    if (FontWorkCheckValidRange(fwk, fglyph, lglyph)) {
        fwk->h.minchar = fglyph;
        fwk->h.numchars = lglyph - fglyph + 1;
    }
    s = GUIGroupGetText(fpropgo, 9, GrGetChrtypeForUserEncoding());
    baseline = atoi(s);
    if (baseline >= 0 && baseline < fwk->h.height)
        fwk->h.baseline = atoi(s);
    free(s);
    s = GUIGroupGetText(fpropgo, 11, GrGetChrtypeForUserEncoding());
    fwk->h.ulheight = atoi(s);
    fwk->h.ulpos = fwk->h.height - fwk->h.ulheight;
    free(s);
    s = GUIGroupGetText(fpropgo, 13, GrGetChrtypeForUserEncoding());
    fwk->h.encoding = atoi(s);
    free(s);
}

int FPropDlgRun(void)
{
    if (!fpropdlg) return -1; 

    fpropdlg->result = -1;
    GUIDialogRun(fpropdlg);
    return fpropdlg->result;
}

