/*
 * dlglicen.c --- edit font license
 * 
 * Copyright (c) 2022 Mariano Alvarez Fernandez (malfer@telefonica.net)
 *
 * This file is part of FEdit, a visual font editor for the MGRX font format
 *
 * FEdit is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FEdit is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FEdit. If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <grgui.h>
#include <mgrxcolr.h>
#include "fedit.h"

#define COMMAND_OK           1
#define COMMAND_CANCEL       2

static GUIDialog *licendlg = NULL;
static GUIGroup *licengo = NULL;

static int process_licendlg_event(void *udata, GrEvent *ev)
{
    GUIDialog *d = (GUIDialog *)udata;
    GUIGroup *go = (GUIGroup *)(d->exdata);

    if (ev->type == GREV_COMMAND) {
        switch (ev->p1) {
            case COMMAND_OK :
                d->result = 1;
               return -1;
            case COMMAND_CANCEL :
                d->result = -1;
                return -1;
        }
    }

    return GUIGroupProcessEvent(go, ev);
}

int LicenDlgCreate(void)
{
    if (licendlg) return 1; // dialog already created

    licengo = GUIGroupCreate(3, 0, 0);
    if (licengo == NULL) return 0;
    GUIObjectSetEdit(&(licengo->o[0]), 0, 0, 0, 480, 200, WEBC_WHITE, WEBC_BLACK,
                     GUI_OEDIT_EDITABLE|GUI_OEDIT_WITHVSCB|GUI_OEDIT_WITHHSCB, 10000, "");
    GUIObjectSetButton(&(licengo->o[1]), 1,  90, 220, 140, 40, WEBC_GREEN, WEBC_WHITE, "Ok", COMMAND_OK, 0, 0);
    GUIObjectSetButton(&(licengo->o[2]), 2, 250, 220, 140, 40, WEBC_GREEN, WEBC_WHITE, "Cancel", COMMAND_CANCEL, 0, 0);
    GUIGroupSetSelected(licengo, 0, 0);

    licendlg = GUIGroupDialogCreate("Edit Font License Text",
                                    licengo, process_licendlg_event);
    if (licendlg == NULL) {
        GUIGroupDestroy(licengo);
        licengo = NULL;
        return 0;
    }

    return 1;
}

void LicenDlgDestroy(void)
{
    if (licendlg) {
        GUIDialogDestroy(licendlg);
        licendlg = NULL;
    }
    if (licengo) {
        GUIGroupDestroy(licengo);
        licengo = NULL;
    }
}

void LicenDlgSetData(char *s)
{
    if (!licendlg) return;

    if (s == NULL) return;

    GUIGroupSetText(licengo, 0, s);
}

void LicenDlgGetData(char **s)
{
    if (!licendlg) return;

    *s = GUIGroupGetText(licengo, 0, GrGetChrtypeForUserEncoding());
}

int LicenDlgRun(void)
{
    if (!licendlg) return -1; 

    licendlg->result = -1;
    GUIDialogRun(licendlg);
    return licendlg->result;
}

