/*
 * dlgwrang.c --- width range dialog
 * 
 * Copyright (c) 2019 Mariano Alvarez Fernandez (malfer@telefonica.net)
 *
 * This file is part of FEdit, a visual font editor for the MGRX font format
 *
 * FEdit is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FEdit is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FEdit. If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <grgui.h>
#include <mgrxcolr.h>
#include "fedit.h"

#define COMMAND_OK           1
#define COMMAND_CANCEL       2

static GUIDialog *wrangdlg = NULL;
static GUIGroup *wranggo = NULL;
static int glb_option = 0;

static int process_wrangdlg_event(void *udata, GrEvent *ev)
{
    GUIDialog *d = (GUIDialog *)udata;
    GUIGroup *go = (GUIGroup *)(d->exdata);

    if (ev->type == GREV_COMMAND) {
        switch (ev->p1) {
            case COMMAND_OK :
                d->result = 1;
               return -1;
            case COMMAND_CANCEL :
                d->result = -1;
                return -1;
        }
    } else if (ev->type == GREV_FCHANGE && ev->p1 == 7) {
        glb_option = ev->p2;
    }

    return GUIGroupProcessEvent(go, ev);
}

int WRangDlgCreate(void)
{
    static char *listopt[2] = {"No", "Yes"};

    if (wrangdlg) return 1; // dialog already created

    wranggo = GUIGroupCreate(10, 0, 0);
    if (wranggo == NULL) return 0;
    GUIObjectSetLabel(&(wranggo->o[0]), 0,   0,   0, 160, 30, GrNOCOLOR, WEBC_WHITE, "First source glyph");
    GUIObjectSetEntry(&(wranggo->o[1]), 1, 160,   0, 160, 30, WEBC_WHITE, WEBC_BLACK, 8, "");
    GUIObjectSetLabel(&(wranggo->o[2]), 2,   0,  40, 160, 30, GrNOCOLOR, WEBC_WHITE, "Last source glyph");
    GUIObjectSetEntry(&(wranggo->o[3]), 3, 160,  40, 160, 30, WEBC_WHITE, WEBC_BLACK, 8, "");
    GUIObjectSetLabel(&(wranggo->o[4]), 4,   0,  80, 160, 30, GrNOCOLOR, WEBC_WHITE, "New width");
    GUIObjectSetEntry(&(wranggo->o[5]), 5, 160,  80, 160, 30, WEBC_WHITE, WEBC_BLACK, 8, "");
    GUIObjectSetLabel(&(wranggo->o[6]), 6,   0, 120, 160, 30, GrNOCOLOR, WEBC_WHITE, "Only blank glyphs");
    GUIObjectSetList(&(wranggo->o[7]),  7, 160, 120, 160, 30, WEBC_WHITE, WEBC_BLACK, (void **)listopt, 2, 2, glb_option);
    GUIObjectSetButton(&(wranggo->o[8]), 8,  10,170, 140, 40, WEBC_GREEN, WEBC_WHITE, "Ok", COMMAND_OK, 0, 0);
    GUIObjectSetButton(&(wranggo->o[9]), 9, 170,170, 140, 40, WEBC_GREEN, WEBC_WHITE, "Cancel", COMMAND_CANCEL, 0, 0);
    GUIGroupSetSelected(wranggo, 1, 0);

    wrangdlg = GUIGroupDialogCreate("Change width glyph in FLrange",
                                    wranggo, process_wrangdlg_event);
    if (wrangdlg == NULL) {
        GUIGroupDestroy(wranggo);
        wranggo = NULL;
        return 0;
    }

    return 1;
}

void WRangDlgDestroy(void)
{
    if (wrangdlg) {
        GUIDialogDestroy(wrangdlg);
        wrangdlg = NULL;
    }
    if (wranggo) {
        GUIGroupDestroy(wranggo);
        wranggo = NULL;
    }
}

void WRangDlgSetData(int fsglyph, int lsglyph, int width)
{
    char s[121];
    
    if (!wrangdlg) return;

    sprintf(s, "%d", fsglyph);
    GUIGroupSetText(wranggo, 1, s);
    sprintf(s, "%d", lsglyph);
    GUIGroupSetText(wranggo, 3, s);
    sprintf(s, "%d", width);
    GUIGroupSetText(wranggo, 5, s);
}

void WRangDlgGetData(int *fsglyph, int *lsglyph, int *width, int *option)
{
    char *s;

    if (!wrangdlg) return;

    s = GUIGroupGetText(wranggo, 1, GrGetChrtypeForUserEncoding());
    *fsglyph = atoi(s);
    free(s);
    s = GUIGroupGetText(wranggo, 3, GrGetChrtypeForUserEncoding());
    *lsglyph = atoi(s);
    free(s);
    s = GUIGroupGetText(wranggo, 5, GrGetChrtypeForUserEncoding());
    *width = atoi(s);
    free(s);
    *option = glb_option;
}

int WRangDlgRun(void)
{
    if (!wrangdlg) return -1; 

    wrangdlg->result = -1;
    GUIDialogRun(wrangdlg);
    return wrangdlg->result;
}

